@@ -22,95 +22,26 @@
 /* printf like formatting for m4.  */
 
 #include "m4.h"
+#include "xvasprintf.h"
 
 /* Simple varargs substitute.  We assume int and unsigned int are the
-   same size; likewise for long and unsigned long.  We do not yet
-   handle long double or long long.  */
+   same size; likewise for long and unsigned long.  */
 
-/* Parse STR as an integer, reporting warnings on behalf of ME.  */
-static int
-arg_int (const char *me, const char *str)
-{
-  char *endp;
-  long value;
+#define ARG_INT(argc, argv) \
+	((argc == 0) ? 0 : \
+	 (--argc, argv++, atoi (TOKEN_DATA_TEXT (argv[-1]))))
 
-  /* TODO - also allow parsing `'a' or `"a' which results in the
-     numeric value of 'a', as in printf(1).  */
-  if (*str == '\0')
-    {
-      m4_warn (0, me, _("empty string treated as 0"));
-      return 0;
-    }
-  errno = 0;
-  value = strtol (str, &endp, 10);
-  if (*endp != '\0')
-    m4_warn (0, me, _("non-numeric argument `%s'"), str);
-  else if (isspace (to_uchar (*str)))
-    m4_warn (0, me, _("leading whitespace ignored"));
-  else if (errno == ERANGE || (int) value != value)
-    m4_warn (0, me, _("numeric overflow detected"));
-  return value;
-}
+#define ARG_LONG(argc, argv) \
+	((argc == 0) ? 0 : \
+	 (--argc, argv++, atol (TOKEN_DATA_TEXT (argv[-1]))))
 
-/* Parse STR as a long, reporting warnings on behalf of ME.  */
-static long
-arg_long (const char *me, const char *str)
-{
-  char *endp;
-  long value;
+#define ARG_STR(argc, argv) \
+	((argc == 0) ? "" : \
+	 (--argc, argv++, TOKEN_DATA_TEXT (argv[-1])))
 
-  /* TODO - also allow parsing `'a' or `"a' which results in the
-     numeric value of 'a', as in printf(1).  */
-  if (*str == '\0')
-    {
-      m4_warn (0, me, _("empty string treated as 0"));
-      return 0L;
-    }
-  errno = 0;
-  value = strtol (str, &endp, 10);
-  if (*endp != '\0')
-    m4_warn (0, me, _("non-numeric argument `%s'"), str);
-  else if (isspace (to_uchar (*str)))
-    m4_warn (0, me, _("leading whitespace ignored"));
-  else if (errno == ERANGE)
-    m4_warn (0, me, _("numeric overflow detected"));
-  return value;
-}
-
-/* Parse STR as a double, reporting warnings on behalf of ME.  */
-static double
-arg_double (const char *me, const char *str)
-{
-  char *endp;
-  double value;
-
-  if (*str == '\0')
-    {
-      m4_warn (0, me, _("empty string treated as 0"));
-      return 0.0;
-    }
-  errno = 0;
-  value = strtod (str, &endp);
-  if (*endp != '\0')
-    m4_warn (0, me, _("non-numeric argument `%s'"), str);
-  else if (isspace (to_uchar (*str)))
-    m4_warn (0, me, _("leading whitespace ignored"));
-  else if (errno == ERANGE)
-    m4_warn (0, me, _("numeric overflow detected"));
-  return value;
-}
-
-#define ARG_INT(i, argc, argv)						\
-  ((argc <= ++i) ? 0 : arg_int (me, ARG (i)))
-
-#define ARG_LONG(i, argc, argv)						\
-  ((argc <= ++i) ? 0L : arg_long (me, ARG (i)))
-
-#define ARG_STR(i, argc, argv)						\
-  ((argc <= ++i) ? "" : ARG (i))
-
-#define ARG_DOUBLE(i, argc, argv)					\
-  ((argc <= ++i) ? 0.0 : arg_double (me, ARG (i)))
+#define ARG_DOUBLE(argc, argv) \
+	((argc == 0) ? 0 : \
+	 (--argc, argv++, strtod (TOKEN_DATA_TEXT (argv[-1]), NULL)))
 
 
 /*------------------------------------------------------------------.
@@ -122,33 +53,30 @@ arg_double (const char *me, const char *str)
 `------------------------------------------------------------------*/
 
 void
-format (struct obstack *obs, int argc, macro_arguments *argv)
+expand_format (struct obstack *obs, int argc, token_data **argv)
 {
-  const char *me = ARG (0);		/* Macro name.  */
-  const char *f;			/* Format control string.  */
-  const char *fmt;			/* Position within f.  */
-  char fstart[] = "%'+- 0#*.*hhd";	/* Current format spec.  */
-  char *p;				/* Position within fstart.  */
-  unsigned char c;			/* A simple character.  */
-  int index = 0;			/* Index within argc used so far.  */
-  bool valid_format = true;		/* True if entire format string ok.  */
+  const char *f;			/* format control string */
+  const char *fmt;			/* position within f */
+  char fstart[] = "%'+- 0#*.*hhd";	/* current format spec */
+  char *p;				/* position within fstart */
+  unsigned char c;			/* a simple character */
 
   /* Flags.  */
-  char flags;				/* Flags to use in fstart.  */
+  char flags;				/* flags to use in fstart */
   enum {
-    THOUSANDS	= 0x01, /* '\''. */
-    PLUS	= 0x02, /* '+'. */
-    MINUS	= 0x04, /* '-'. */
-    SPACE	= 0x08, /* ' '. */
-    ZERO	= 0x10, /* '0'. */
-    ALT		= 0x20, /* '#'. */
-    DONE	= 0x40  /* No more flags.  */
+    THOUSANDS	= 0x01, /* ' */
+    PLUS	= 0x02, /* + */
+    MINUS	= 0x04, /* - */
+    SPACE	= 0x08, /*   */
+    ZERO	= 0x10, /* 0 */
+    ALT		= 0x20, /* # */
+    DONE	= 0x40  /* no more flags */
   };
 
   /* Precision specifiers.  */
-  int width;			/* Minimum field width.  */
-  int prec;			/* Precision.  */
-  char lflag;			/* Long flag.  */
+  int width;			/* minimum field width */
+  int prec;			/* precision */
+  char lflag;			/* long flag */
 
   /* Specifiers we are willing to accept.  ok['x'] implies %x is ok.
      Various modifiers reduce the set, in order to avoid undefined
@@ -156,23 +84,17 @@ format (struct obstack *obs, int argc, macro_arguments *argv)
   char ok[128];
 
   /* Buffer and stuff.  */
-  char *base;			/* Current position in obs.  */
-  size_t len;			/* Length of formatted text.  */
-  char *str;			/* Malloc'd buffer of formatted text.  */
+  char *str;			/* malloc'd buffer of formatted text */
   enum {CHAR, INT, LONG, DOUBLE, STR} datatype;
 
-  f = fmt = ARG_STR (index, argc, argv);
+  f = fmt = ARG_STR (argc, argv);
   memset (ok, 0, sizeof ok);
-  while (true)
+  for (;;)
     {
       while ((c = *fmt++) != '%')
 	{
 	  if (c == '\0')
-	    {
-	      if (valid_format)
-		bad_argc (me, argc, index, index);
-	      return;
-	    }
+	    return;
 	  obstack_1grow (obs, c);
 	}
 
@@ -250,7 +172,7 @@ format (struct obstack *obs, int argc, macro_arguments *argv)
       *p++ = '*';
       if (*fmt == '*')
 	{
-	  width = ARG_INT (index, argc, argv);
+	  width = ARG_INT (argc, argv);
 	  fmt++;
 	}
       else
@@ -270,7 +192,7 @@ format (struct obstack *obs, int argc, macro_arguments *argv)
 	  ok['c'] = 0;
 	  if (*(++fmt) == '*')
 	    {
-	      prec = ARG_INT (index, argc, argv);
+	      prec = ARG_INT (argc, argv);
 	      ++fmt;
 	    }
 	  else
@@ -308,8 +230,8 @@ format (struct obstack *obs, int argc, macro_arguments *argv)
       c = *fmt++;
       if (c > sizeof ok || !ok[c])
 	{
-	  m4_warn (0, me, _("unrecognized specifier in `%s'"), f);
-	  valid_format = false;
+	  M4ERROR ((warning_status, 0,
+		    "Warning: unrecognized specifier in `%s'", f));
 	  if (c == '\0')
 	    fmt--;
 	  continue;
@@ -352,56 +274,39 @@ format (struct obstack *obs, int argc, macro_arguments *argv)
 	}
       *p++ = c;
       *p = '\0';
-      base = obstack_next_free (obs);
-      len = obstack_room (obs);
 
       switch (datatype)
 	{
 	case CHAR:
-	  str = asnprintf (base, &len, fstart, width,
-			   ARG_INT (index, argc, argv));
+	  str = xasprintf (fstart, width, ARG_INT(argc, argv));
 	  break;
 
 	case INT:
-	  str = asnprintf (base, &len, fstart, width, prec,
-			   ARG_INT (index, argc, argv));
+	  str = xasprintf (fstart, width, prec, ARG_INT(argc, argv));
 	  break;
 
 	case LONG:
-	  str = asnprintf (base, &len, fstart, width, prec,
-			   ARG_LONG (index, argc, argv));
+	  str = xasprintf (fstart, width, prec, ARG_LONG(argc, argv));
 	  break;
 
 	case DOUBLE:
-	  str = asnprintf (base, &len, fstart, width, prec,
-			   ARG_DOUBLE (index, argc, argv));
+	  str = xasprintf (fstart, width, prec, ARG_DOUBLE(argc, argv));
 	  break;
 
 	case STR:
-	  str = asnprintf (base, &len, fstart, width, prec,
-			   ARG_STR (index, argc, argv));
+	  str = xasprintf (fstart, width, prec, ARG_STR(argc, argv));
 	  break;
 
 	default:
-	  abort ();
+	  abort();
 	}
 
+      /* NULL was returned on failure, such as invalid format string.  For
+	 now, just silently ignore that bad specifier.  */
       if (str == NULL)
-	/* NULL is unexpected (EILSEQ and EINVAL are not possible
-	   based on our construction of fstart, leaving only ENOMEM,
-	   which should always be fatal).  */
-	m4_error (EXIT_FAILURE, errno, me,
-		  _("unable to format output for `%s'"), f);
-      else if (str == base)
-	/* The output was already computed in place, but we need to
-	   account for its size.  */
-	obstack_blank_fast (obs, len);
-      else
-	{
-	  /* The output exceeded available obstack space, copy the
-	     allocated string.  */
-	  obstack_grow (obs, str, len);
-	  free (str);
-	}
+	continue;
+
+      obstack_grow (obs, str, strlen (str));
+      free (str);
     }
 }
