@@ -1,9 +1,9 @@
 /* Test of closein module.
-   Copyright (C) 2007 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include "binary-io.h"
 
@@ -35,15 +36,17 @@ int
 main (int argc, char **argv)
 {
   char buf[7];
-  int i = -1;
-  atexit(close_stdin);
+  atexit (close_stdin);
   program_name = argv[0];
 
   /* close_stdin currently relies on ftell, but mingw ftell is
      unreliable on text mode input.  */
   SET_BINARY (0);
 
+  if (argc > 2)
+    close (0);
+
   if (argc > 1)
-    i = fread (buf, 1, 6, stdin);
+    fread (buf, 1, 6, stdin);
   return 0;
 }
